@@ -344,17 +344,29 @@ NTSTATUS security_descriptor_sacl_del(struct security_descriptor *sd,
 /*
   compare two security ace structures
 */
-bool security_ace_equal(const struct security_ace *ace1, 
+bool security_ace_equal(const struct security_ace *ace1,
 			const struct security_ace *ace2)
 {
-	if (ace1 == ace2) return true;
-	if (!ace1 || !ace2) return false;
-	if (ace1->type != ace2->type) return false;
-	if (ace1->flags != ace2->flags) return false;
-	if (ace1->access_mask != ace2->access_mask) return false;
-	if (!dom_sid_equal(&ace1->trustee, &ace2->trustee)) return false;
+	if (ace1 == ace2) {
+		return true;
+	}
+	if ((ace1 == NULL) || (ace2 == NULL)) {
+		return false;
+	}
+	if (ace1->type != ace2->type) {
+		return false;
+	}
+	if (ace1->flags != ace2->flags) {
+		return false;
+	}
+	if (ace1->access_mask != ace2->access_mask) {
+		return false;
+	}
+	if (!dom_sid_equal(&ace1->trustee, &ace2->trustee)) {
+		return false;
+	}
 
-	return true;	
+	return true;
 }
 
 
@@ -564,24 +576,44 @@ struct security_ace *security_ace_create(TALLOC_CTX *mem_ctx,
 					 uint8_t flags)
 
 {
-	struct dom_sid *sid;
 	struct security_ace *ace;
+	bool ok;
 
 	ace = talloc_zero(mem_ctx, struct security_ace);
 	if (ace == NULL) {
 		return NULL;
 	}
 
-	sid = dom_sid_parse_talloc(ace, sid_str);
-	if (sid == NULL) {
+	ok = dom_sid_parse(sid_str, &ace->trustee);
+	if (!ok) {
 		talloc_free(ace);
 		return NULL;
 	}
-
-	ace->trustee = *sid;
 	ace->type = type;
 	ace->access_mask = access_mask;
 	ace->flags = flags;
 
 	return ace;
 }
+
+/*******************************************************************
+ Check for MS NFS ACEs in a sd
+*******************************************************************/
+bool security_descriptor_with_ms_nfs(const struct security_descriptor *psd)
+{
+	int i;
+
+	if (psd->dacl == NULL) {
+		return false;
+	}
+
+	for (i = 0; i < psd->dacl->num_aces; i++) {
+		if (dom_sid_compare_domain(
+			    &global_sid_Unix_NFS,
+			    &psd->dacl->aces[i].trustee) == 0) {
+			return true;
+		}
+	}
+
+	return false;
+}
