@@ -20,17 +20,12 @@
 #include "includes.h"
 #include "torture/proto.h"
 
-#ifdef CLUSTER_SUPPORT
-
 #include "ctdb_conn.h"
+#include "ctdbd_conn.h"
 #include "lib/util/tevent_unix.h"
 #include "tdb.h"
 
-#ifdef HAVE_CTDB_PROTOCOL_H
 #include "ctdb_protocol.h"
-#else
-#include "ctdb_private.h"
-#endif
 
 #include "messages.h"
 
@@ -221,7 +216,7 @@ bool run_ctdb_conn(int dummy)
 	struct tevent_req *req;
 	int ret;
 
-	ev = tevent_context_init(talloc_tos());
+	ev = samba_tevent_context_init(talloc_tos());
 	if (ev == NULL) {
 		fprintf(stderr, "tevent_context_init failed\n");
 		return false;
@@ -242,12 +237,3 @@ bool run_ctdb_conn(int dummy)
 	TALLOC_FREE(ev);
 	return (ret == 0);
 }
-
-#else /* CLUSTER_SUPPORT */
-
-bool run_ctdb_conn(int dummy)
-{
-	return true;
-}
-
-#endif
