@@ -7,16 +7,18 @@
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "includes.h"
+#include "system/filesys.h"
+#include "popt_common.h"
 #include "libsmbclient.h"
 
 #if _FILE_OFFSET_BITS==64
@@ -49,7 +51,8 @@ static const char *username = NULL, *password = NULL, *workgroup = NULL;
 static int nonprompt = 0, quiet = 0, dots = 0, keep_permissions = 0, verbose = 0, send_stdout = 0;
 static int blocksize = SMB_DEFAULT_BLOCKSIZE;
 
-static int smb_download_file(const char *base, const char *name, int recursive, int resume, char *outfile);
+static int smb_download_file(const char *base, const char *name, int recursive,
+			     int resume, int toplevel, char *outfile);
 
 static int get_num_cols(void)
 {
@@ -74,10 +77,15 @@ static void change_columns(int sig)
 
 static void human_readable(off_t s, char *buffer, int l)
 {
-	if(s > 1024 * 1024 * 1024) snprintf(buffer, l, "%.2fGb", 1.0 * s / (1024 * 1024 * 1024));
-	else if(s > 1024 * 1024) snprintf(buffer, l, "%.2fMb", 1.0 * s / (1024 * 1024));
-	else if(s > 1024) snprintf(buffer, l, "%.2fkb", 1.0 * s / 1024);
-	else snprintf(buffer, l, OFF_T_FORMAT"b", (OFF_T_FORMAT_CAST)s);
+	if (s > 1024 * 1024 * 1024) {
+		snprintf(buffer, l, "%.2fGB", 1.0 * s / (1024 * 1024 * 1024));
+	} else if (s > 1024 * 1024) {
+		snprintf(buffer, l, "%.2fMB", 1.0 * s / (1024 * 1024));
+	} else if (s > 1024) {
+		snprintf(buffer, l, "%.2fkB", 1.0 * s / 1024);
+	} else {
+		snprintf(buffer, l, OFF_T_FORMAT"b", (OFF_T_FORMAT_CAST)s);
+	}
 }
 
 static void get_auth_data(const char *srv, const char *shr, char *wg, int wglen, char *un, int unlen, char *pw, int pwlen)
@@ -101,13 +109,12 @@ static void get_auth_data(const char *srv, const char *shr, char *wg, int wglen,
 	} else if(username) strncpy(un, username, unlen-1);
 
 	if(!nonprompt && !password) {
-		char *prompt, *pass;
+		char *prompt;
 		if (asprintf(&prompt, "Password for %s at %s: ", shr, srv) == -1) {
 			return;
 		}
-		pass = getpass(prompt);
+		(void) samba_getpass(prompt, pw, pwlen, false, false);
 		free(prompt);
-		strncpy(pw, pass, pwlen-1);
 	} else if(password) strncpy(pw, password, pwlen-1);
 
 	if(workgroup)strncpy(wg, workgroup, wglen-1);
@@ -135,20 +142,24 @@ static int smb_download_dir(const char *base, const char *name, int resume)
 	/* List files in directory and call smb_download_file on them */
 	dirhandle = smbc_opendir(path);
 	if(dirhandle < 1) {
-		if(errno == ENOTDIR) return smb_download_file(base, name, 1, resume, NULL);
+		if (errno == ENOTDIR) {
+			return smb_download_file(base, name, 1, resume,
+						 0, NULL);
+		}
 		fprintf(stderr, "Can't open directory %s: %s\n", path, strerror(errno));
 		return 1;
 	}
 
 	while(*relname == '/')relname++;
 	mkdir(relname, 0755);
-	
+
 	tmpname = SMB_STRDUP(name);
 
 	while((dirent = smbc_readdir(dirhandle))) {
 		char *newname;
 		if(!strcmp(dirent->name, ".") || !strcmp(dirent->name, ".."))continue;
 		if (asprintf(&newname, "%s/%s", tmpname, dirent->name) == -1) {
+			free(tmpname);
 			return 1;
 		}
 		switch(dirent->smbc_type) {
@@ -165,7 +176,8 @@ static int smb_download_dir(const char *base, const char *name, int resume)
 			break;
 
 		case SMBC_FILE:
-			ret = smb_download_file(base, newname, 1, resume, NULL);
+			ret = smb_download_file(base, newname, 1, resume, 0,
+						NULL);
 			break;
 
 		case SMBC_FILE_SHARE:
@@ -179,7 +191,7 @@ static int smb_download_dir(const char *base, const char *name, int resume)
 		case SMBC_COMMS_SHARE:
 			if(!quiet)printf("Ignoring comms share %s\n", dirent->name);
 			break;
-			
+
 		case SMBC_IPC_SHARE:
 			if(!quiet)printf("Ignoring ipc$ share %s\n", dirent->name);
 			break;
@@ -198,7 +210,7 @@ static int smb_download_dir(const char *base, const char *name, int resume)
 			smbc_closedir(dirhandle);
 			return 1;
 		}
-		
+
 		if(chmod(relname, remotestat.st_mode) < 0) {
 			fprintf(stderr, "Unable to change mode of local dir %s to %o\n", relname,
 				(unsigned int)remotestat.st_mode);
@@ -247,7 +259,7 @@ static void print_progress(const char *name, time_t start, time_t now, off_t sta
 	if (len == -1) {
 		return;
 	}
-	
+
 	if(columns) {
 		int required = strlen(name), available = columns - len - strlen("[] ");
 		if(required > available) {
@@ -266,9 +278,11 @@ static void print_progress(const char *name, time_t start, time_t now, off_t sta
 
 /* Return 1 on error, 0 on success. */
 
-static int smb_download_file(const char *base, const char *name, int recursive, int resume, char *outfile) {
+static int smb_download_file(const char *base, const char *name, int recursive,
+			     int resume, int toplevel, char *outfile)
+{
 	int remotehandle, localhandle;
-	time_t start_time = time(NULL);
+	time_t start_time = time_mono(NULL);
 	const char *newpath;
 	char path[SMB_MAXPATHLEN];
 	char checkbuf[2][RESUME_CHECK_SIZE];
@@ -277,7 +291,7 @@ static int smb_download_file(const char *base, const char *name, int recursive,
 	struct stat localstat, remotestat;
 
 	snprintf(path, SMB_MAXPATHLEN-1, "%s%s%s", base, (*base && *name && name[0] != '/' && base[strlen(base)-1] != '/')?"/":"", name);
-	
+
 	remotehandle = smbc_open(path, O_RDONLY, 0755);
 
 	if(remotehandle < 0) {
@@ -322,8 +336,10 @@ static int smb_download_file(const char *base, const char *name, int recursive,
 		if(newpath)newpath++; else newpath = base;
 	} else newpath = name;
 
-	if(newpath[0] == '/')newpath++;
-	
+	if (!toplevel && (newpath[0] == '/')) {
+		newpath++;
+	}
+
 	/* Open local file according to the mode */
 	if(update) {
 		/* if it is up-to-date, skip */
@@ -350,7 +366,7 @@ static int smb_download_file(const char *base, const char *name, int recursive,
 			smbc_close(remotehandle);
 			return 1;
 		}
-	
+
 		if (fstat(localhandle, &localstat) != 0) {
 			fprintf(stderr, "Can't fstat %s: %s\n", newpath, strerror(errno));
 			smbc_close(remotehandle);
@@ -401,6 +417,7 @@ static int smb_download_file(const char *base, const char *name, int recursive,
 				fprintf(stderr, "Offset in local and remote files is different (local: "OFF_T_FORMAT", remote: "OFF_T_FORMAT")\n",
 					(OFF_T_FORMAT_CAST)off1,
 					(OFF_T_FORMAT_CAST)off2);
+				smbc_close(remotehandle); close(localhandle);
 				return 1;
 			}
 
@@ -432,6 +449,12 @@ static int smb_download_file(const char *base, const char *name, int recursive,
 	}
 
 	readbuf = (char *)SMB_MALLOC(blocksize);
+	if (!readbuf) {
+		if (localhandle != STDOUT_FILENO) {
+			close(localhandle);
+		}
+		return 1;
+	}
 
 	/* Now, download all bytes from offset_download to the end */
 	for(curpos = offset_download; curpos < remotestat.st_size; curpos+=blocksize) {
@@ -456,7 +479,8 @@ static int smb_download_file(const char *base, const char *name, int recursive,
 
 		if(dots)fputc('.', stderr);
 		else if(!quiet) {
-			print_progress(newpath, start_time, time(NULL), start_offset, curpos, remotestat.st_size);
+			print_progress(newpath, start_time, time_mono(NULL),
+					start_offset, curpos, remotestat.st_size);
 		}
 	}
 
@@ -496,7 +520,7 @@ static void clean_exit(void)
 	char bs[100];
 	human_readable(total_bytes, bs, sizeof(bs));
 	if(!quiet)fprintf(stderr, "Downloaded %s in %lu seconds\n", bs,
-		(unsigned long)(time(NULL) - total_start_time));
+		(unsigned long)(time_mono(NULL) - total_start_time));
 	exit(0);
 }
 
@@ -649,17 +673,18 @@ int main(int argc, const char **argv)
 	if (smb_encrypt) {
 		SMBCCTX *smb_ctx = smbc_set_context(NULL);
 		smbc_option_set(smb_ctx,
-			CONST_DISCARD(char *, "smb_encrypt_level"),
+			discard_const_p(char, "smb_encrypt_level"),
 			"require");
 	}
-	
+
 	columns = get_num_cols();
 
-	total_start_time = time(NULL);
+	total_start_time = time_mono(NULL);
 
 	while ( (file = poptGetArg(pc)) ) {
 		if (!recursive) 
-			ret = smb_download_file(file, "", recursive, resume, outputfile);
+			ret = smb_download_file(file, "", recursive, resume,
+						1, outputfile);
 		else 
 			ret = smb_download_dir(file, "", resume);
 	}
