@@ -22,8 +22,10 @@
 #ifndef __LIBCLI_RAW_INTERFACES_H__
 #define __LIBCLI_RAW_INTERFACES_H__
 
-#include "smb.h" 
+#include "source4/libcli/raw/smb.h"
+#include "../libcli/smb/smb_common.h"
 #include "librpc/gen_ndr/misc.h" /* for struct GUID */
+#include "librpc/gen_ndr/smb2_lease_struct.h"
 
 /* this structure is just a wrapper for a string, the only reason we
    bother with this is that it allows us to check the length provided
@@ -53,6 +55,16 @@ struct smb2_handle {
 	uint64_t data[2];
 };
 
+struct smb2_lease_break {
+	struct smb2_lease current_lease;
+	uint16_t new_epoch; /* only for v2 leases */
+	uint32_t break_flags;
+	uint32_t new_lease_state;
+	uint32_t break_reason; /* should be 0 */
+	uint32_t access_mask_hint; /* should be 0 */
+	uint32_t share_mask_hint; /* should be 0 */
+};
+
 struct ntvfs_handle;
 
 /*
@@ -161,7 +173,7 @@ union smb_mkdir {
 		enum smb_mkdir_level level;
 		struct {
 			const char *path;
-			uint_t num_eas;
+			unsigned int num_eas;
 			struct ea_struct *eas;			
 		} in;
 	} t2mkdir;
@@ -259,6 +271,8 @@ union smb_tcon {
 		} in;
 		struct {
 			uint16_t options;
+			uint32_t max_access;
+			uint32_t guest_max_access;
 			char *dev_type;
 			char *fs_type;
 			uint16_t tid;
@@ -464,7 +478,7 @@ union smb_fileinfo {
 		struct {
 			uint32_t attrib;
 			uint32_t ea_size;
-			uint_t num_eas;
+			unsigned int num_eas;
 			struct ea_struct {
 				uint8_t flags;
 				struct smb_wire_string name;
@@ -492,7 +506,7 @@ union smb_fileinfo {
 			uint32_t mode;
 			uint32_t alignment_requirement;
 			uint32_t reparse_tag;
-			uint_t num_streams;
+			unsigned int num_streams;
 			struct stream_struct {
 				uint64_t size;
 				uint64_t alloc_size;
@@ -554,14 +568,14 @@ union smb_fileinfo {
 		enum smb_fileinfo_level level;
 		struct {
 			union smb_handle_or_path file;
-			uint_t num_names;
+			unsigned int num_names;
 			struct ea_name {
 				struct smb_wire_string name;
 			} *ea_names;	
 		} in;	
 
 		struct smb_ea_list {
-			uint_t num_eas;
+			unsigned int num_eas;
 			struct ea_struct *eas;
 		} out;
 	} ea_list;
@@ -708,7 +722,7 @@ union smb_fileinfo {
 			union smb_handle_or_path file;
 		} in;
 		struct stream_information {
-			uint_t num_streams;
+			unsigned int num_streams;
 			struct stream_struct *streams;
 		} out;
 	} stream_info;
@@ -900,8 +914,8 @@ enum smb_setfileinfo_level {
 	RAW_SFILEINFO_END_OF_FILE_INFO        = SMB_SFILEINFO_END_OF_FILE_INFO,
 	RAW_SFILEINFO_UNIX_BASIC              = SMB_SFILEINFO_UNIX_BASIC,
 	RAW_SFILEINFO_UNIX_INFO2              = SMB_SFILEINFO_UNIX_INFO2,
-	RAW_SFILEINFO_UNIX_LINK               = SMB_SFILEINFO_UNIX_LINK,
-	RAW_SFILEINFO_UNIX_HLINK	      = SMB_SFILEINFO_UNIX_HLINK,
+	RAW_SFILEINFO_UNIX_LINK               = SMB_SET_FILE_UNIX_LINK,
+	RAW_SFILEINFO_UNIX_HLINK	      = SMB_SET_FILE_UNIX_HLINK,
 	RAW_SFILEINFO_BASIC_INFORMATION       = SMB_SFILEINFO_BASIC_INFORMATION,
 	RAW_SFILEINFO_RENAME_INFORMATION      = SMB_SFILEINFO_RENAME_INFORMATION,
 	RAW_SFILEINFO_LINK_INFORMATION        = SMB_SFILEINFO_LINK_INFORMATION,
@@ -969,7 +983,7 @@ union smb_setfileinfo {
 		enum smb_setfileinfo_level level;
 		struct {
 			union smb_handle_or_path file;
-			uint_t num_eas;
+			unsigned int num_eas;
 			struct ea_struct *eas;			
 		} in;
 	} ea_set;
@@ -1116,7 +1130,7 @@ union smb_setfileinfo {
 		} in;
 	} unix_link, unix_hlink;
 
-	/* RAW_FILEINFO_SET_SEC_DESC */
+	/* RAW_SFILEINFO_SEC_DESC */
 	struct {
 		enum smb_setfileinfo_level level;
 		struct {
@@ -1320,6 +1334,26 @@ union smb_fsinfo {
 };
 
 
+enum smb_setfsinfo_level {
+		RAW_SETFS_UNIX_INFO                      = SMB_SET_CIFS_UNIX_INFO};
+
+union smb_setfsinfo {
+	/* generic interface */
+	struct {
+		enum smb_fsinfo_level level;
+	} generic;
+
+	/* TRANS2 RAW_QFS_UNIX_INFO interface */
+	struct {
+		enum smb_fsinfo_level level;
+
+		struct {
+			uint16_t major_version;
+			uint16_t minor_version;
+			uint64_t capability;
+		} in;
+	} unix_info;
+};
 
 enum smb_open_level {
 	RAW_OPEN_OPEN,
@@ -1332,6 +1366,7 @@ enum smb_open_level {
 	RAW_OPEN_T2OPEN,
 	RAW_OPEN_NTTRANS_CREATE, 
 	RAW_OPEN_OPENX_READX,
+	RAW_OPEN_NTCREATEX_READX,
 	RAW_OPEN_SMB2
 };
 
@@ -1376,6 +1411,9 @@ union smb_open {
 	case RAW_OPEN_OPENX_READX: \
 		file = &op->openxreadx.out.file; \
 		break; \
+	case RAW_OPEN_NTCREATEX_READX: \
+		file = &op->ntcreatexreadx.out.file; \
+		break; \
 	case RAW_OPEN_SMB2: \
 		file = &op->smb2.out.file; \
 		break; \
@@ -1390,7 +1428,7 @@ union smb_open {
 		enum smb_open_level level;
 		struct {
 			uint32_t flags;
-			uint32_t root_fid;
+			union smb_handle root_fid;
 			uint32_t access_mask;
 			uint64_t alloc_size;
 			uint32_t file_attr;
@@ -1411,6 +1449,9 @@ union smb_open {
 			
 			/* some optional parameters from the SMB2 varient */
 			bool query_maximal_access;
+
+			/* private flags for internal use only */
+			uint8_t private_flags;
 		} in;
 		struct {
 			union smb_handle file;
@@ -1446,7 +1487,7 @@ union smb_open {
 			uint32_t size;
 			uint32_t timeout;
 			const char *fname;
-			uint_t num_eas;
+			unsigned int num_eas;
 			struct ea_struct *eas;			
 		} in;
 		struct {
@@ -1595,6 +1636,54 @@ union smb_open {
 		} out;
 	} openxreadx;
 
+	/* chained NTCreateX/ReadX interface */
+	struct {
+		enum smb_open_level level;
+		struct {
+			uint32_t flags;
+			union smb_handle root_fid;
+			uint32_t access_mask;
+			uint64_t alloc_size;
+			uint32_t file_attr;
+			uint32_t share_access;
+			uint32_t open_disposition;
+			uint32_t create_options;
+			uint32_t impersonation;
+			uint8_t  security_flags;
+			/* NOTE: fname can also be a pointer to a
+			 uint64_t file_id if create_options has the
+			 NTCREATEX_OPTIONS_OPEN_BY_FILE_ID flag set */
+			const char *fname;
+
+			/* readx part */
+			uint64_t offset;
+			uint16_t mincnt;
+			uint32_t maxcnt;
+			uint16_t remaining;
+		} in;
+		struct {
+			union smb_handle file;
+			uint8_t oplock_level;
+			uint32_t create_action;
+			NTTIME create_time;
+			NTTIME access_time;
+			NTTIME write_time;
+			NTTIME change_time;
+			uint32_t attrib;
+			uint64_t alloc_size;
+			uint64_t size;
+			uint16_t file_type;
+			uint16_t ipc_state;
+			uint8_t  is_directory;
+
+			/* readx part */
+			uint8_t *data;
+			uint16_t remaining;
+			uint16_t compaction_mode;
+			uint16_t nread;
+		} out;
+	} ntcreatexreadx;
+
 #define SMB2_CREATE_FLAG_REQUEST_OPLOCK           0x0100
 #define SMB2_CREATE_FLAG_REQUEST_EXCLUSIVE_OPLOCK 0x0800
 #define SMB2_CREATE_FLAG_GRANT_OPLOCK             0x0001
@@ -1630,18 +1719,24 @@ union smb_open {
 			struct security_descriptor *sec_desc;
 			bool   durable_open;
 			struct smb2_handle *durable_handle;
+
+			/* data for durable handle v2 */
+			bool durable_open_v2;
+			struct GUID create_guid;
+			bool persistent_open;
+			uint32_t timeout;
+			struct smb2_handle *durable_handle_v2;
+
 			bool   query_maximal_access;
 			NTTIME timewarp;
 			bool   query_on_disk_id;
-			
+			struct smb2_lease *lease_request;
+			struct smb2_lease *lease_request_v2;
+
+			struct GUID *app_instance_id;
+
 			/* and any additional blobs the caller wants */
-			struct smb2_create_blobs {
-				uint32_t num_blobs;
-				struct smb2_create_blob {
-					const char *tag;
-					DATA_BLOB data;
-				} *blobs;
-			} blobs;
+			struct smb2_create_blobs blobs;
 		} in;
 		struct {
 			union smb_handle file;
@@ -1666,6 +1761,14 @@ union smb_open {
 			/* optional return values matching tagged values in the call */
 			uint32_t maximal_access;
 			uint8_t on_disk_id[32];
+			struct smb2_lease lease_response;
+			struct smb2_lease lease_response_v2;
+			bool durable_open;
+
+			/* durable handle v2 */
+			bool durable_open_v2;
+			bool persistent_open;
+			uint32_t timeout;
 
 			/* tagged blobs in the reply */
 			struct smb2_create_blobs blobs;
@@ -1707,6 +1810,8 @@ union smb_read {
 			uint16_t remaining;
 			uint16_t compaction_mode;
 			uint32_t nread;
+			uint16_t flags2;
+			uint16_t data_offset;
 		} out;
 	} readx, generic;
 
@@ -1958,18 +2063,11 @@ union smb_lock {
 			/* static body buffer 48 (0x30) bytes */
 			/* uint16_t buffer_code;  0x30 */
 			uint16_t lock_count;
-			uint32_t reserved;
+			uint32_t lock_sequence;
 			/* struct smb2_handle handle; */
 			struct smb2_lock_element {
 				uint64_t offset;
 				uint64_t length;
-/* these flags are the same as the SMB2 lock flags */
-#define SMB2_LOCK_FLAG_NONE		0x00000000
-#define SMB2_LOCK_FLAG_SHARED		0x00000001
-#define SMB2_LOCK_FLAG_EXCLUSIVE	0x00000002
-#define SMB2_LOCK_FLAG_UNLOCK		0x00000004
-#define SMB2_LOCK_FLAG_FAIL_IMMEDIATELY	0x00000010
-#define SMB2_LOCK_FLAG_ALL_MASK		0x00000017
 				uint32_t flags;
 				uint32_t reserved;
 			} *locks;
@@ -1994,6 +2092,14 @@ union smb_lock {
 			/* struct smb2_handle handle; */
 		} in, out;
 	} smb2_break;
+
+	/* SMB2 Lease Break Ack (same opcode as smb2_break) */
+	struct smb2_lease_break_ack {
+		struct {
+			uint32_t reserved;
+			struct smb2_lease lease;
+		} in, out;
+	} smb2_lease_break_ack;
 };
 
 
@@ -2014,7 +2120,6 @@ union smb_close {
 		struct {
 			union smb_handle file;
 			time_t write_time;
-#define SMB2_CLOSE_FLAGS_FULL_INFORMATION (1<<0)
 			uint16_t flags; /* SMB2_CLOSE_FLAGS_* */
 		} in;
 		struct {
@@ -2411,7 +2516,7 @@ union smb_search_first {
 			const char *pattern;
 
 			/* the ea names are only used for RAW_SEARCH_EA_LIST */
-			uint_t num_names;
+			unsigned int num_names;
 			struct ea_name *ea_names;
 		} in;
 		struct {
@@ -2421,22 +2526,6 @@ union smb_search_first {
 		} out;
 	} t2ffirst;
 
-/*
-  SMB2 uses different level numbers for the same old SMB trans2 search levels
-*/
-#define SMB2_FIND_DIRECTORY_INFO         0x01
-#define SMB2_FIND_FULL_DIRECTORY_INFO    0x02
-#define SMB2_FIND_BOTH_DIRECTORY_INFO    0x03
-#define SMB2_FIND_NAME_INFO              0x0C
-#define SMB2_FIND_ID_BOTH_DIRECTORY_INFO 0x25
-#define SMB2_FIND_ID_FULL_DIRECTORY_INFO 0x26
-
-/* flags for SMB2 find */
-#define SMB2_CONTINUE_FLAG_RESTART    0x01
-#define SMB2_CONTINUE_FLAG_SINGLE     0x02
-#define SMB2_CONTINUE_FLAG_INDEX      0x04
-#define SMB2_CONTINUE_FLAG_REOPEN     0x10
-
 	/* SMB2 Find */
 	struct smb2_find {
 		enum smb_search_level level;
@@ -2511,7 +2600,7 @@ union smb_search_next {
 			const char *last_name;
 
 			/* the ea names are only used for RAW_SEARCH_EA_LIST */
-			uint_t num_names;
+			unsigned int num_names;
 			struct ea_name *ea_names;
 		} in;
 		struct {
@@ -2651,6 +2740,7 @@ union smb_search_data {
 		uint32_t  attrib;
 		uint32_t  ea_size;
 		uint64_t file_id;
+		uint8_t short_name_buf[24];
 		struct smb_wire_string short_name;
 		struct smb_wire_string name;
 	} id_both_directory_info;
